from fastapi.responses import FileResponse, HTMLResponse, Response
from markdown import markdown
from textile import textile
from docutils.core import publish_parts
import os


def wrap_html(inner_html):
    return HTMLResponse(
        f"<!DOCTYPE html><html><body>{inner_html}</body></html>"
    )


def serve_meta(converter):
    def server_wrapper(file_path):
        async def server(request, script_directory):
            with open(
                os.path.join(script_directory, file_path),
                "r",
                encoding="utf-8"
            ) as file:
                return converter(file.read())
        return server
    return server_wrapper


serve_rst = serve_meta(lambda text: wrap_html(
    publish_parts(text, writer_name="html")["html_body"]
))
serve_markdown = serve_meta(lambda text: wrap_html(markdown(text)))
serve_textile = serve_meta(lambda text: wrap_html(textile(text)))
serve_html = serve_meta(lambda text: HTMLResponse(text))
serve_file = serve_meta(lambda text: FileResponse(text))
serve_text = serve_meta(lambda text: Response(text))



from fastapi import FastAPI, Request, Response
from fastapi.responses import FileResponse
from http import HTTPStatus
import uvicorn
import argparse
import os
import re

parser = argparse.ArgumentParser(description=(
    "A dead simple server for personal websites. "
    "Launch in a directory that needs to be served."
))
parser.add_argument("--port", default=443, type=int)
parser.add_argument("--log-level", default="INFO")
args = parser.parse_args()

MULTISLASH = re.compile(r"/{2,}")

app = FastAPI()


@app.api_route("{file_path:path}")
async def serve(request: Request, file_path: str):
    if "/../" in file_path or file_path.endswith("/.."):
        return Response("Don't go down.", status_code=HTTPStatus.FORBIDDEN)
    file_path = MULTISLASH.sub("/", file_path)
    file_path = file_path[1:]  # Removing the root slash
    if os.path.isdir(file_path):
        script_path = os.path.join(file_path, "page.py")
        if os.path.isfile(script_path):
            script_locals = {}
            with open(script_path, "r", encoding="utf-8") as script:
                exec(script.read(), {}, script_locals)
            try:
                return await (
                    script_locals[request.method.lower()](request, file_path)
                )
            except KeyError:
                return Response(
                    "Method not allowed.",
                    status_code=HTTPStatus.METHOD_NOT_ALLOWED,
                )
    elif os.path.isfile(file_path):
        return FileResponse(file_path)
    return Response("Page not found.", status_code=HTTPStatus.NOT_FOUND)


uvicorn.run(app, port=args.port, log_level=args.log_level.lower())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['little_server', 'little_server.utils']

package_data = \
{'': ['*']}

install_requires = \
['Markdown>=3.4.1,<4.0.0',
 'docutils>=0.19,<0.20',
 'fastapi>=0.81.0,<0.82.0',
 'textile>=4.0.2,<5.0.0',
 'uvicorn>=0.18.3,<0.19.0']

setup_kwargs = {
    'name': 'little-server',
    'version': '2.0.0',
    'description': 'A dead simple server for personal websites',
    'long_description': None,
    'author': 'megahomyak',
    'author_email': 'g.megahomyak@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7',
}


setup(**setup_kwargs)
